import { Body, Controller, Get, HttpCode, NotFoundException, Param, ParseIntPipe, Patch, Post, Req, UseGuards } from '@nestjs/common'
import { UsersService } from './users.service'
import { ApiBadRequestResponse, ApiBearerAuth, ApiForbiddenResponse, ApiNoContentResponse, ApiNotFoundResponse, ApiOkResponse, ApiUnauthorizedResponse } from '@nestjs/swagger'
import { UserEntity } from './entities/user.entity'
import { JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { User } from '@prisma/client'
import { UpdatePasswordDto } from './dto/user_password.dto'

export type AuthenticatedRequest = Request & { user: User }

@Controller('users')
export class UsersController {
  constructor(private readonly usersService: UsersService) {}

  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: UserEntity, isArray: true })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async findAll() {
    const users = await this.usersService.findAll()
    return users.map(user => new UserEntity(user))
  }

  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: UserEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Utilisateur⋅rice non trouvé⋅e" })
  async findOne(@Param('id', ParseIntPipe) id: number) {
    const user = await this.usersService.findOne(id)
    if (!user)
      throw new NotFoundException(`L'utilisateur⋅rice avec l'identifiant ${id} n'existe pas`)
    return new UserEntity(user)
  }

  @Patch('/update-password')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiNoContentResponse({description: "Le mot de passe a bien été modifié."})
  @ApiBadRequestResponse({description: "Erreur dans la saisie du nouveau mot de passe."})
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async updatePassword(@Req() request: AuthenticatedRequest, @Body() { password }: UpdatePasswordDto) {
    const user = request.user
    await this.usersService.updatePassword(user, password)
  }
}
